using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using NppAwkPluginCS;

namespace NppAwkPluginCS.Forms
{
    public partial class NppAwkPluginMainForm : Form
    {
        public NppAwkPluginMainForm()
        {
            InitializeComponent();
        }

        private List<AwkScriptXml> moListeScripts = null;
        private AwkScriptXml moSelectedScript = null;


        private void B_ExecScript_Click(object sender, EventArgs e)
        {
            Microsoft.Win32.RegistryKey okey = Microsoft.Win32.Registry.CurrentUser.OpenSubKey(@"Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders");
            string sUserAppDataPath = okey.GetValue("AppData") as string;
            string sNppAwkPluginPath = Path.Combine(sUserAppDataPath, "NppAwkPlugin");
            string sAwkPath = TB_PathAwk.Text;

            string sNppAwkPluginDataPath = Path.Combine(sNppAwkPluginPath, "NppAwkPluginData.txt");
            string sNppAwkPluginScriptPath = Path.Combine(sNppAwkPluginPath, "NppAwkPluginScript.txt");
            string sNppAwkPluginResultPath = Path.Combine(sNppAwkPluginPath, "NppAwkPluginResult.txt");

            if (!Directory.Exists(sNppAwkPluginPath))
            {
                Directory.CreateDirectory(sNppAwkPluginPath);
            }


            StreamWriter oStreamData = new StreamWriter(sNppAwkPluginDataPath, false, Encoding.GetEncoding("ISO-8859-1"));

            try
            {
                oStreamData.Write(TB_Data.Text);
                oStreamData.Flush();
            }
            finally
            {
                oStreamData.Close();
            }

            StreamWriter oStreamScript = new StreamWriter(sNppAwkPluginScriptPath, false, Encoding.GetEncoding("ISO-8859-1"));
            try
            {
                oStreamScript.Write(TB_ScriptAwk.Text);
                oStreamScript.Flush();
            }
            finally
            {
                oStreamScript.Close();
            }

            System.Diagnostics.ProcessStartInfo oInfo = new System.Diagnostics.ProcessStartInfo(sAwkPath, string.Format("-f {0} {1} > {2}", sNppAwkPluginScriptPath, sNppAwkPluginDataPath, sNppAwkPluginResultPath));
            oInfo.UseShellExecute = false;
            oInfo.RedirectStandardOutput = true;
            oInfo.CreateNoWindow = true;
            System.Diagnostics.Process oProcess = new System.Diagnostics.Process();
            oProcess.StartInfo = oInfo;
            oProcess.OutputDataReceived += new System.Diagnostics.DataReceivedEventHandler(oProcess_OutputDataReceived);

            oOutputProcess = new List<string>();
            oProcess.Start();
            oProcess.BeginOutputReadLine();
            try
            {
                oProcess.WaitForExit();
                TB_Result.Clear();
                foreach (string sLine in oOutputProcess)
                {
                    if (sLine != null)
                    {
                        TB_Result.AppendText(sLine);
                        TB_Result.AppendText(Environment.NewLine);
                    }
                }
            }
            finally
            {
                oProcess.Close();
            }
        }

        private List<string> oOutputProcess = null;
        void oProcess_OutputDataReceived(object sender, System.Diagnostics.DataReceivedEventArgs e)
        {

            oOutputProcess.Add(e.Data);


        }

        private void BtnSelectAwkPath_Click(object sender, EventArgs e)
        {
            openFileDialog1.ShowDialog();
        }

        void openFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            TB_PathAwk.Text = openFileDialog1.FileName;
            NppAwkPluginConfig.PluginConfig.AwkPath = openFileDialog1.FileName;
            NppAwkPluginConfig.WriteConfig();
        }

        private void NppAwkPluginMainForm_Load(object sender, EventArgs e)
        {
             

            //load scripts;
            moListeScripts = NppAwkPluginConfig.GetAwkScripts();
            if (moListeScripts.Count == 0)
            {
                moListeScripts.Add(NewScript());
            }
            foreach(AwkScriptXml oScript in moListeScripts) {
                oScript.IsModified = false;
            }
            CBScriptTitle.Items.AddRange(moListeScripts.ToArray());
            CBScriptTitle.SelectedItem = moListeScripts[0];

            TB_PathAwk.Text = NppAwkPluginConfig.PluginConfig.AwkPath;
        }

        private void CBScriptTitle_SelectedIndexChanged(object sender, EventArgs e)
        {
            AwkScriptXml oSelectedScript = CBScriptTitle.SelectedItem as AwkScriptXml;
            if (oSelectedScript != null)
            {
                TB_ScriptAwk.Text = oSelectedScript.Script;
                moSelectedScript = oSelectedScript;
            }
        }

        private void BtnNewScript_Click(object sender, EventArgs e)
        {

            NewScript();
            //create new script
            AwkScriptXml oNewScript = NewScript();
            moListeScripts.Add(oNewScript);
            int iIndex = CBScriptTitle.Items.Add(oNewScript);
            CBScriptTitle.SelectedIndex = iIndex;
 
        }

        private AwkScriptXml NewScript()
        {
            //create new script
            int iIdScript = 0;
            foreach (AwkScriptXml oItem in moListeScripts)
            {
                iIdScript = Math.Max(iIdScript, oItem.Id);
            }
            iIdScript++;
            AwkScriptXml oNewScript = new AwkScriptXml();
            oNewScript.Title = string.Format("Untitled {0}", iIdScript);
            oNewScript.Id = iIdScript;
            oNewScript.IsModified = false;

            return oNewScript;
        }


        private void BtnDeleteScript_Click(object sender, EventArgs e)
        {
            AwkScriptXml oSelectedScript = CBScriptTitle.SelectedItem as AwkScriptXml;
            if (oSelectedScript != null)
            {
                NppAwkPluginConfig.DeleteAwkScript(oSelectedScript);
                int iIndex = CBScriptTitle.Items.IndexOf(oSelectedScript);
                moListeScripts.Remove(oSelectedScript);
                CBScriptTitle.Items.Remove(oSelectedScript);
                moSelectedScript = null;

                if (CBScriptTitle.Items.Count == 0)
                {
                    moListeScripts.Add(NewScript());
                    CBScriptTitle.Items.AddRange(moListeScripts.ToArray());
                }
                CBScriptTitle.SelectedIndex = Math.Max(iIndex-1, 0);
            }
        }

        private void TB_ScriptAwk_Validating(object sender, CancelEventArgs e)
        {
            //save current script
            AwkScriptXml oSelectedScript = CBScriptTitle.SelectedItem as AwkScriptXml;
            if (oSelectedScript != null)
            {
                oSelectedScript.Script = TB_ScriptAwk.Text;
            }
        }


        private void CBScriptTitle_Validating(object sender, CancelEventArgs e)
        {
            //save current script
            AwkScriptXml oSelectedScriptInCombo = CBScriptTitle.SelectedItem as AwkScriptXml;
            if (moSelectedScript != null && oSelectedScriptInCombo == null)
            {
                moSelectedScript.Title = CBScriptTitle.Text;
                int iIndex = CBScriptTitle.Items.IndexOf(moSelectedScript);
                CBScriptTitle.Items.RemoveAt(iIndex);
                CBScriptTitle.Items.Insert(iIndex, moSelectedScript);
            }
        }

        private void NppAwkPluginMainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            foreach (AwkScriptXml oScript in moListeScripts)
            {
                if (oScript.IsModified)
                {
                    NppAwkPluginConfig.WriteAwkScript(oScript);
                }
            }
        }


        private void BtnCopyFromNpp_Click(object sender, EventArgs e)
        {
            if (this.CopyFromNpp != null)
            {
                this.CopyFromNpp(sender, e);
            }
        }

        private void BtnCopyToNpp_Click(object sender, EventArgs e)
        {
            if (this.CopyToNpp != null)
            {
                this.CopyToNpp(sender, e);
            }
        }


        public EventHandler CopyFromNpp;
        public EventHandler CopyToNpp;
        public void SetDataString(string sData)
        {
            TB_Data.Text = sData;
        }

        public string GetResultString()
        {
            return TB_Result.Text;
        }
        
    }
}