using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace NppAwkPluginCS
{
    [ClassInterface(ClassInterfaceType.None)]
    [Guid("B90D444F-23D5-4C21-AC72-066C431B9D61")]
    public class NppAwkPlugin : INppPlugin
    {
        private NppData moNppData;
        private List<FuncItem> moTab = new List<FuncItem>();
        private IntPtr[] moTabIntPtr = null;
        private NppCommands moNppCommands = null;
        private NppAwkPluginCS.Forms.NppAwkPluginMainForm moWindow;

        #region constructor / destructor
        // Need a public default constructor for COM Interop.
        public NppAwkPlugin() { }

        ~NppAwkPlugin()
        {
            //destructor
            foreach (IntPtr oElt in moTabIntPtr)
            {
                if (oElt != null)
                {
                    Marshal.FreeCoTaskMem(oElt);
                }
            }
        }
        #endregion

        #region DllMain


        public void DllMain(ulong iReasonForCall)
        {

            switch (iReasonForCall)
            {
                // Here we initialize the FuncItem Array.
                // AWARE : no need to initialize _cmdID field.
                // This filed will be initialized by Notepad++ plugins system
                case NppConst.DLL_PROCESS_ATTACH:
                    {

                        FuncItem oElt = null;


                        oElt = new FuncItem();
                        oElt.ItemName = "Run Awk4N++...";
                        oElt.pFunc = new TestDelegate(ShowMainWindow);
                        oElt.Init2Check = false;
                        oElt.PShKey = new ShortcutKey();
                        oElt.PShKey.IsAlt = true;
                        oElt.PShKey.IsCtrl = true;
                        oElt.PShKey.Key = 0x41; //VK_A
                        moTab.Add(oElt);


                        oElt = new FuncItem();
                        oElt.ItemName = "About...";
                        oElt.pFunc = new TestDelegate(About);
                        oElt.Init2Check = false;
                        oElt.PShKey = null;
                        moTab.Add(oElt);

                    }
                    break;
                case NppConst.DLL_PROCESS_DETACH:
                    break;
                case NppConst.DLL_THREAD_ATTACH:
                    break;
                case NppConst.DLL_THREAD_DETACH:
                    break;
            }

        }

        #endregion

        #region Mandatory functions


        public void setInfo(NppData oNppData)
        {

            moNppData = oNppData;
            moNppCommands = new NppCommands(moNppData);
        }

        public void getName(out string sName)
        {

            sName = "Awk for N++";

        }

        public void beNotified(SCNotification oNotification)
        {
            //nothing
        }

        public void messageProc(uint Message, IntPtr wParam, IntPtr lParam)
        {
            //nothing
        }

        public void getFuncsArray(out int iLen, out IntPtr[] oTab)
        {

            iLen = moTab.Count;
            moTabIntPtr = new IntPtr[iLen];


            for (int i = 0; i < iLen; i++)
            {
                moTabIntPtr[i] = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(FuncItem)));
                Marshal.StructureToPtr(moTab[i], moTabIntPtr[i], false);
            }

            oTab = moTabIntPtr;

        }

        #endregion

        #region INppPlugin Membres


        public void About()
        {
            NppAwkPluginCS.Forms.AboutForm oAboutForm = new NppAwkPluginCS.Forms.AboutForm();
            oAboutForm.ShowDialog(new NppWindowHandle(moNppData.NppHandle));
        }

        public void ShowMainWindow()
        {
            moWindow = new NppAwkPluginCS.Forms.NppAwkPluginMainForm();
            moWindow.CopyFromNpp += new EventHandler(CopyFromNppHandler);
            moWindow.CopyToNpp += new EventHandler(CopyToNppHandler);
            moWindow.Show(new NppWindowHandle(moNppData.NppHandle));
        }

        private void CopyFromNppHandler(object sender, EventArgs e)
        {
            moWindow.SetDataString(
                moNppCommands.ScintillaGetSelectedText(moNppCommands.GetCurrentScintillaHandle())
                );
        }

        private void CopyToNppHandler(object sender, EventArgs e)
        {
            moNppCommands.ScintillaReplaceSelection(moNppCommands.GetCurrentScintillaHandle(), moWindow.GetResultString());
        }

        #endregion
    }
}
