using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.Xml;
using System.IO;

namespace NppAwkPluginCS
{
    class NppAwkPluginConfig
    {
        private const string NPP_AWK_PLUGIN_CONFIG_FILE = "NppAwkPluginConfig.xml";

        private static string goNppAwkPluginPath = null;
        private static NppAwkPluginConfigXml goNppConfig = null;

        public static string NppAwkPluginPath
        {
            get
            {
                if (goNppAwkPluginPath == null)
                {
                    Microsoft.Win32.RegistryKey okey = Microsoft.Win32.Registry.CurrentUser.OpenSubKey(@"Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders");
                    string sUserAppDataPath = okey.GetValue("AppData") as string;
                    goNppAwkPluginPath = Path.Combine(sUserAppDataPath, "NppAwkPlugin");
                }
                return goNppAwkPluginPath;
            }
        }


        /// <summary>read plugin configuration</summary>
        public static NppAwkPluginConfigXml PluginConfig
        {
            get
            {
                if (goNppConfig == null)
                {

                    string sConfigFilePath = Path.Combine(NppAwkPluginPath, NPP_AWK_PLUGIN_CONFIG_FILE);
                    if (!File.Exists(sConfigFilePath))
                    {
                        goNppConfig = new NppAwkPluginConfigXml();
                        goNppConfig.AwkPath = "gawk.exe";
                        goNppConfig.Version = System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.ToString();
                        
                        WriteConfig();
                    }
                    else
                    {
                        XmlSerializer oSerializer = new XmlSerializer(typeof(NppAwkPluginConfigXml));
                        XmlTextReader oReader = new XmlTextReader(sConfigFilePath);
                        try
                        {
                            goNppConfig = (NppAwkPluginConfigXml)oSerializer.Deserialize(oReader);
                        }
                        finally
                        {
                            oReader.Close();
                        }
                    }
                }

                return goNppConfig;
            }
        }


        /// <summary>write plugin configuration</summary>
        /// <param name="oConfig">plugin config</param>
        public static void WriteConfig()
        {
            if (goNppConfig != null)
            {
                XmlSerializer oSerializer = new XmlSerializer(typeof(NppAwkPluginConfigXml));
                XmlTextWriter oWriter = new XmlTextWriter(Path.Combine(NppAwkPluginPath, NPP_AWK_PLUGIN_CONFIG_FILE),Encoding.UTF8);
                try
                {
                    oSerializer.Serialize(oWriter, goNppConfig);
                }
                finally
                {
                    oWriter.Close();
                }
            }
        }

        /// <summary>Read awk scripts</summary>
        /// <returns>list of scripts</returns>
        public static List<AwkScriptXml> GetAwkScripts()
        {
            List<AwkScriptXml> oAwkScriptsList = new List<AwkScriptXml>();
            XmlSerializer oSerializer = new XmlSerializer(typeof(AwkScriptXml));
            foreach (string sFile in Directory.GetFiles(NppAwkPluginPath, "*.script.xml"))
            {
                XmlTextReader oReader = new XmlTextReader(sFile);
                try
                {
                    AwkScriptXml oScript = oSerializer.Deserialize(oReader) as AwkScriptXml;
                    oAwkScriptsList.Add(oScript);
                }
                finally
                {
                    oReader.Close();
                }
            }
            return oAwkScriptsList;
        }

        public static void WriteAwkScripts(List<AwkScriptXml> oScripts) {
            
            foreach (AwkScriptXml oScript in oScripts)
            {
                WriteAwkScript(oScript);
            }
        }

        public static void WriteAwkScript(AwkScriptXml oScript)
        {
            XmlSerializer oSerializer = new XmlSerializer(typeof(AwkScriptXml));
            XmlTextWriter oWriter = new XmlTextWriter(Path.Combine(NppAwkPluginPath, string.Format("script{0:000}.script.xml", oScript.Id)),Encoding.UTF8);
            try
            {
                oSerializer.Serialize(oWriter, oScript);
            }
            finally
            {
                oWriter.Close();
            }
        }

        public static void DeleteAwkScript(AwkScriptXml oScript)
        {
            File.Delete(Path.Combine(NppAwkPluginPath, string.Format("script{0:000}.script.xml", oScript.Id)));
        }
    }
}
